__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2012, Canari Project'
__credits__ = []

__license__ = 'GPL'
__version__ = '1.1'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'

__all__ = [
    'commands',
    'maltego',
    'pkgutils',
    'resources',
    'tds',
    'utils',
    'xmltools',
    'config',
    'easygui',
    'framework',
    'resource'
]